@@ -1,7 +1,8 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2014 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2012-2018 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2016-2018 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   Portions of this file are adapted from Alacarte Menu Editor,
 #   Copyright (C) 2006 Travis Watkins, Heinrich Wendel
@@ -25,10 +26,16 @@ import xml.parsers.expat
 from locale import gettext as _
 from xml.sax.saxutils import escape
 
+import logging
+logger = logging.getLogger('menulibre') # noqa
+
+import gi
+gi.require_version('GMenu', '3.0')  # noqa
+
 from gi.repository import GdkPixbuf, Gio, GLib, GMenu, Gtk
 
 from . import util
-from .util import MenuItemTypes
+from .util import MenuItemTypes, escapeText
 
 locale.textdomain('menulibre')
 
@@ -36,9 +43,6 @@ icon_theme = Gtk.IconTheme.get_default()
 
 menu_name = ""
 
-import logging
-logger = logging.getLogger('menulibre')
-
 
 def get_default_menu():
     """Return the filename of the default application menu."""
@@ -83,22 +87,25 @@ def menu_to_treestore(treestore, parent, menu_items):
         item_type = item[0]
         if item_type == MenuItemTypes.SEPARATOR:
             displayed_name = "<s>                    </s>"
+            # Translators: Separator menu item
             tooltip = _("Separator")
+            categories = ""
             filename = None
             icon = None
             icon_name = ""
+            show = True
         else:
             displayed_name = escape(item[2]['display_name'])
-            if not item[2]['show']:
-                displayed_name = "<small><i>%s</i></small>" % displayed_name
-            tooltip = item[2]['comment']
+            show = item[2]['show']
+            tooltip = escapeText(item[2]['comment'])
+            categories = item[2]['categories']
             icon = item[2]['icon']
             filename = item[2]['filename']
             icon_name = item[2]['icon_name']
 
         treeiter = treestore.append(
-            parent, [displayed_name, tooltip, item_type,
-            icon, icon_name, filename, False])
+            parent, [displayed_name, tooltip, categories, item_type,
+                     icon, icon_name, filename, False, show])
 
         if item_type == MenuItemTypes.DIRECTORY:
             treestore = menu_to_treestore(treestore, treeiter, item[3])
@@ -108,8 +115,10 @@ def menu_to_treestore(treestore, parent, menu_items):
 
 def get_treestore():
     """Get the TreeStore implementation of the current menu."""
-    # Name, Comment, MenuItemType, GIcon (TreeView), icon-name, Filename, exp
-    treestore = Gtk.TreeStore(str, str, int, Gio.Icon, str, str, bool)
+    # Name, Comment, Categories, MenuItemType, GIcon (TreeView), icon-name,
+    # Filename, exp, show
+    treestore = Gtk.TreeStore(str, str, str, int, Gio.Icon, str, str, bool,
+                              bool)
     menu = get_menus()[0]
     return menu_to_treestore(treestore, None, menu)
 
@@ -126,28 +135,34 @@ def get_submenus(menu, tree_dir):
                 entry_id = child.get_desktop_file_id()
                 app_info = child.get_app_info()
                 icon = app_info.get_icon()
-                icon_name = "application-default-icon"
+                icon_name = "applications-other"
                 display_name = app_info.get_display_name()
                 generic_name = app_info.get_generic_name()
                 comment = app_info.get_description()
                 keywords = app_info.get_keywords()
+                categories = app_info.get_categories()
                 executable = app_info.get_executable()
                 filename = child.get_desktop_file_path()
-                hidden = app_info.get_is_hidden()
                 submenus = None
 
+                is_hidden = app_info.get_is_hidden()
+                no_display = app_info.get_nodisplay()
+                show_in = app_info.get_show_in()
+                hidden = is_hidden or no_display or not show_in
+
             elif isinstance(child, GMenu.TreeDirectory):
                 item_type = MenuItemTypes.DIRECTORY
                 entry_id = child.get_menu_id()
                 icon = child.get_icon()
-                icon_name = "application-default-icon"
+                icon_name = "folder"
                 display_name = child.get_name()
                 generic_name = child.get_generic_name()
                 comment = child.get_comment()
                 keywords = []
+                categories = ""
                 executable = None
                 filename = child.get_desktop_file_path()
-                hidden = False
+                hidden = child.get_is_nodisplay()
                 submenus = get_submenus(menu, child)
 
             if isinstance(icon, Gio.ThemedIcon):
@@ -159,6 +174,7 @@ def get_submenus(menu, tree_dir):
                        'generic_name': generic_name,
                        'comment': comment,
                        'keywords': keywords,
+                       'categories': categories,
                        'executable': executable,
                        'filename': filename,
                        'icon': icon,
@@ -216,14 +232,17 @@ class MenuEditor(object):
 
     def __init__(self, basename=None):
         """init"""
+
+        # Remember to keep menulibre-menu-validate's GMenu object creation
+        # in-sync with this code
         basename = basename or get_default_menu()
 
         self.tree = GMenu.Tree.new(basename,
-                                    GMenu.TreeFlags.SHOW_EMPTY |
-                                    GMenu.TreeFlags.INCLUDE_EXCLUDED |
-                                    GMenu.TreeFlags.INCLUDE_NODISPLAY |
-                                    GMenu.TreeFlags.SHOW_ALL_SEPARATORS |
-                                    GMenu.TreeFlags.SORT_DISPLAY_NAME)
+                                   GMenu.TreeFlags.SHOW_EMPTY |
+                                   GMenu.TreeFlags.INCLUDE_EXCLUDED |
+                                   GMenu.TreeFlags.INCLUDE_NODISPLAY |
+                                   GMenu.TreeFlags.SHOW_ALL_SEPARATORS |
+                                   GMenu.TreeFlags.SORT_DISPLAY_NAME)
         self.load()
 
         self.path = os.path.join(
