@@ -1,7 +1,8 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2014 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2012-2018 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2016 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   This program is free software: you can redistribute it and/or modify it
 #   under the terms of the GNU General Public License version 3, as published
@@ -15,14 +16,14 @@
 #   You should have received a copy of the GNU General Public License along
 #   with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-#lint:disable
+# lint:disable
 try:
     import xml.etree.cElementTree
     from xml.etree.cElementTree import ElementTree, Element, SubElement
 except ImportError:
-    import xml.etree.ElementTree
+    import xml.etree.ElementTree  # noqa
     from xml.etree.ElementTree import ElementTree, Element, SubElement
-#lint:enable
+# lint:enable
 
 import os
 
@@ -202,8 +203,8 @@ def model_to_xml_menus(model, model_parent=None, menu_parent=None):
         treeiter = model.iter_nth_child(model_parent, n_child)
 
         # Extract the menu item details.
-        name, comment, item_type, gicon, icon, desktop, expanded = \
-                model[treeiter][:]
+        name, comment, categories, item_type, gicon, icon, desktop, expanded, \
+            show = model[treeiter][:]
 
         if item_type == MenuItemTypes.DIRECTORY:
             # Do not save duplicate directories.
@@ -225,6 +226,10 @@ def model_to_xml_menus(model, model_parent=None, menu_parent=None):
             # Do Menus
             model_to_xml_menus(model, treeiter, next_element)
 
+            # Do Includes to allow for alacarte-created entries without
+            # categories to persist (see LP: #1315880)
+            model_to_xml_includes(model, treeiter, next_element)
+
             # Do Layouts
             model_to_xml_layout(model, treeiter, next_element)
 
@@ -235,7 +240,43 @@ def model_to_xml_menus(model, model_parent=None, menu_parent=None):
             pass
 
 
-def model_to_xml_layout(model, model_parent=None, menu_parent=None, merge=True):
+def model_to_xml_includes(model, model_parent=None, menu_parent=None):
+    """Append <Include> elements for any application items that lack categories
+    in a system directory (e.g. alacarte-created entries), and all items in
+    custom directories."""
+
+    # Looping for all items in directory
+    for n_child in range(model.iter_n_children(model_parent)):
+        treeiter = model.iter_nth_child(model_parent, n_child)
+
+        # Extract the menu item details.
+        name, comment, categories, item_type, gicon, icon, desktop, expanded, \
+            show = model[treeiter][:]
+
+        if desktop is None:
+            continue
+
+        # Detecting custom user directories
+        user_directory = False
+        if model_parent and categories:
+            for category in categories.split(';'):
+                if category.startswith('menulibre-'):
+                    user_directory = True
+                    break
+
+        # Items in custom directories by menulibre have a category, but
+        # includes are required otherwise they are dropped by GMenu
+        if item_type == MenuItemTypes.APPLICATION and (
+                not categories or user_directory):
+            include = menu_parent.addInclude()
+            try:
+                include.addFilename(os.path.basename(desktop))
+            except AttributeError:
+                pass
+
+
+def model_to_xml_layout(model, model_parent=None, menu_parent=None,  # noqa
+                        merge=True):
     """Append the <Layout> element to menu_parent."""
     layout = menu_parent.addLayout()
 
@@ -247,8 +288,8 @@ def model_to_xml_layout(model, model_parent=None, menu_parent=None, merge=True):
         treeiter = model.iter_nth_child(model_parent, n_child)
 
         # Extract the menu item details.
-        name, comment, item_type, gicon, icon, desktop, expanded = \
-                model[treeiter][:]
+        name, comment, categories, item_type, gicon, icon, desktop, expanded, \
+            show = model[treeiter][:]
 
         if item_type == MenuItemTypes.DIRECTORY:
             # Do not save duplicate directories.
@@ -268,9 +309,24 @@ def model_to_xml_layout(model, model_parent=None, menu_parent=None, merge=True):
                 directory_name = util.getDirectoryName(desktop)
                 layout.addMenuname(directory_name)
 
-        elif item_type == MenuItemTypes.APPLICATION:
+        elif item_type == MenuItemTypes.APPLICATION and desktop is not None:
             try:
-                layout.addFilename(os.path.basename(desktop))
+
+                # According to the spec, desktop files may be located in
+                # subdirectories of the '*/applications' directory that
+                # effectively gives the contained desktop filenames an extra
+                # prefix based on that directory name (this affects me with the
+                # kde4 subdirectory desktop files). This only seems to matter
+                # for layout generation - if you don't specify the desktop file
+                # with the prefix here, the prefixed desktop file will not
+                # match in the layout node when the menu is being constructed
+                # See LP: #1315536 comment 5
+                containing_dir = os.path.basename(os.path.dirname(desktop))
+                desktop_filename = os.path.basename(desktop)
+                if containing_dir != 'applications':
+                    desktop_filename = '%s-%s' % (containing_dir,
+                                                  desktop_filename)
+                layout.addFilename(desktop_filename)
             except AttributeError:
                 pass
 
@@ -293,7 +349,11 @@ def model_children_to_xml(model, model_parent=None, menu_parent=None):
     # Menus First...
     model_to_xml_menus(model, model_parent, menu_parent)
 
-    # Layouts Second...
+    # Includes Second... to allow for alacarte-created entries without
+    # categories to persist (see LP: #1315880)
+    model_to_xml_includes(model, model_parent, menu_parent)
+
+    # Layouts Third...
     model_to_xml_layout(model, model_parent, menu_parent, merge=False)
 
 
