@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2015 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2012-2018 Sean Davis <smd.seandavis@gmail.com>
 #
 #   This program is free software: you can redistribute it and/or modify it
 #   under the terms of the GNU General Public License version 3, as published
@@ -18,7 +18,7 @@
 '''facade - makes menulibre_lib package easy to refactor
 
 while keeping its api constant'''
-#lint:disable
-from . helpers import get_builder, set_up_logging, show_uri
-from . menulibreconfig import get_version
-#lint:enable
+# lint:disable
+from . helpers import get_builder, set_up_logging, show_uri  # noqa
+from . menulibreconfig import get_version  # noqa
+# lint:enable
